--DESCRIPTION
--List all the columns for a table, together with
--their datatypes, default values, and nullability.
--
--INPUTS
--  Param 1   A table name, optionally qualified
--            optionally qualified by an owner name.
--            For example: "SYS.ALL_TAB_COLUMNS".
--
SET ECHO OFF
SET VERIFY OFF
SET FEEDBACK OFF
SET RECSEP OFF
SET HEADING ON
SET PAGESIZE 9999
SET LONG 1000
SET LINESIZE 80
CLEAR COLUMNS
CLEAR BREAKS
TTITLE OFF
BTITLE OFF

--Disect the input argument, and get the owner name and
--table name into two seperate substitution variables.
--The owner name defaults to the current user.
DEFINE s_owner_name = ' '
DEFINE s_table_name = ' '
COLUMN owner_name NOPRINT NEW_VALUE s_owner_name
COLUMN table_name NOPRINT NEW_VALUE s_table_name
SELECT
  DECODE(INSTR('&&1','.'),
         0,USER,  /*Default to current user.*/
         UPPER(SUBSTR('&&1',1,INSTR('&&1','.')-1))) owner_name,
  DECODE(INSTR('&&1','.'),
         0,UPPER('&&1'),  /*Only the table name was passed in.*/
         UPPER(SUBSTR('&&1',INSTR('&&1','.')+1))) table_name
  FROM dual;

--Setup the page title, force just one page break by breaking on 
--the table name column.
TTITLE LEFT -
       "COLUMN LISTING FOR " s_owner_name '.' s_table_name -
       SKIP 2
BREAK ON table_name SKIP PAGE

--Setup the formats and widths for the columns to be displayed.
COLUMN column_id NOPRINT
COLUMN table_name NOPRINT
COLUMN column_name FORMAT A30 HEADING 'Column Name'
COLUMN data_type FORMAT A17 HEADING 'Data Type'
COLUMN not_null FORMAT A9 HEADING 'Nullable?'

SELECT table_name,
       column_id,
       column_name,
       DECODE (data_type,
         'VARCHAR2','VARCHAR2 (' || TO_CHAR(data_length) || ')',
         'NVARCHAR2','NVARCHAR2 (' || TO_CHAR(data_length) || ')',
         'CHAR','CHAR (' || TO_CHAR(data_length) || ')',
         'NCHAR','NCHAR (' || TO_CHAR(data_length) || ')',
         'NUMBER',
            DECODE (data_precision,
              NULL, 'NUMBER',
              'NUMBER (' || TO_CHAR(data_precision) 
                         || ',' || TO_CHAR(data_scale) || ')'),
         'FLOAT',
            DECODE (data_precision,
              NULL, 'FLOAT',
              'FLOAT (' || TO_CHAR(data_precision) || ')'),
         'DATE','DATE',
         'LONG','LONG',
         'LONG RAW','LONG RAW',
         'RAW','RAW (' || TO_CHAR(data_length) || ')',
         'MLSLABEL','MLSLABEL',
         'ROWID','ROWID',
         'CLOB','CLOB',
         'NCLOB','NCLOB',
         'BLOB','BLOB',
         'BFILE','BFILE',
         data_type || ' ???') data_type,
         DECODE (nullable, 'N','NOT NULL') not_null
  FROM all_tab_columns
 WHERE table_name = '&&s_table_name'
   AND owner = '&&s_owner_name'
ORDER BY column_id;

--Now list any columns with default values defined.
--This is done as a seporate select because SQL*Plus
--versions 8.0.3 and 8.0.4 do not handle null LONG
--columns properly.
TTITLE SKIP 2 LEFT -
       "DEFAULT VALUES FOR " s_owner_name '.' s_table_name -
       SKIP 2
               
COLUMN default_value HEADING 'Default Value' FORMAT A40 WORD_WRAPPED
SELECT table_name,
       column_name,
       data_default default_value
  FROM all_tab_columns
 WHERE table_name = '&&s_table_name'
   AND owner = '&&s_owner_name'
   AND data_default IS NOT NULL
ORDER BY column_id;

--Clean up by undefining variables, clearing break and 
--column definitions, and setting all settings back to 
--their defaults.
UNDEFINE s_owner_name
UNDEFINE s_table_name

CLEAR BREAKS
CLEAR COLUMNS
TTITLE OFF

SET ECHO OFF
SET VERIFY ON
SET FEEDBACK ON
SET RECSEP WRAPPED
SET HEADING ON
SET PAGESIZE 14
SET LONG 80
SET LINESIZE 80

